# -*- coding: utf-8 -*-
"""
formatage des donnees et utilisation des gabarits Cheetah pour l'affichage
"""
from twisted.python import log
from nevow import loaders, tags as T
from nevow.url import URL
from os import path as osp
from glob import glob

from ead2.frontend.web.lib.layout import TableLayout, SpecialTableLayout
from Cheetah.Template import Template as ChtTemplate
from Cheetah.NameMapper import NotFound as CheetahNotFound
from config import TEMPLATE_DIR
import urllib, commands, traceback

class CheetahTemplate(ChtTemplate):
    """personnalise et facilite la construction
    du template Cheetah
    """
    def __init__(self, filename, context):
        self.context = context
        builtins = {}
        ChtTemplate.__init__(self, file=filename, searchList=[context, builtins, {'is_defined':self.is_defined}])

    def is_defined(self, varname):
        """filtre permettant de ne pas lever d'exception au cas où
           la variable ne serait pas définie
        """
        return varname in self.context

def is_available(template):
    """:Returns:
    - True si le gabarit est disponible
    - False sinon
    """
    if osp.isfile(osp.join(TEMPLATE_DIR, template + '.tmpl')):
        return True
    return False

def write_template(templatename, content):
    """recupere un template s'il n'est pas disponible en local depuis le frontend web
    :templatename: nom du template
    :content: contenu du template
    """
    filename = osp.join(TEMPLATE_DIR,templatename + '.tmpl')
    fh = file(filename, 'w')
    fh.write(content)
    fh.close()
    
def instanciate_template(template, context):
    """instancie un gabarit
    :Returns: la chaine instanciee
    """
    template_name = osp.join(TEMPLATE_DIR,template + '.tmpl')
    try:
        chtpl = CheetahTemplate(template_name, context['data']['content'])
        #chtpl = ChtTemplate(file=template_name, searchList=[context['data']['content'],{}])
        result = str(chtpl)
    except CheetahNotFound, err:
        # utiliser a ce moment la un template par default
        raise TemplateError, "erreur dans l'instanciation du template %s" % template
    return str(result)

def compile_template(template):
    if not is_compiled(template):
        template_name = osp.join(TEMPLATE_DIR,template + '.tmpl')
        cmd = """/usr/bin/cheetah-compile %s"""%template_name
        a,b = commands.getstatusoutput(cmd)

def is_compiled(template):
    compiled_name = osp.join(TEMPLATE_DIR,template + '.py')
    if osp.isfile(compiled_name):
        return True
    return False

        
def format_action(result, proxy, magic_nb):
    """
    :proxy: le backend xml-rpc 
    :result: resultat de la sortie de l'action

    dans le resultat de l'action, un dictionnaire
    comprend:
    
    - une clef 'template' (le gabarit)
    - une clef 'data' (le resultat de l'action proprement dit)

    dans 'data' :
    
        - une clef 'content', qui est un contenu par defaut eventuellement
        - d'autres clefs qui peuvent etre utilisees comme variables de gabarit
    """
    # dictionnaire de vues par defaut en fonction des types de donnees
    res_layout = {list:TableLayout(3)}
    # unescape les caracteres speciaux apres le passage par xml-rpc
    result = urllib.unquote(result)
    # evalutation de la liste des sorties
    try:
        content = eval(result)
        data = content['data']
    except:
        # si ce qui est renvoye n'est pas normalise, c'est une chaine
        content = result
        data = content
    if type(data) == dict:
        if data.has_key('templates'):
            plugins = data['templates']
            for plug in plugins:
                if not is_available(plug):
                    content_plug = proxy.get_template(plug, magic_nb)[1]
                    write_template(plug, content_plug)
                compile_template(plug)
    if type(content) != str: 
        if content.has_key('template'):
            try:
                # le template existe
                template = content['template']
                if is_available(template):
                    # le template est disponible
 #                   compile_template(template)
                    a = instanciate_template(template, content)
                    return T.xml(a)
                else:
                    # le template n'est pas disponible
                    try:
                        # tentative de recuperation du template
                        code, content_template = proxy.get_template(template, magic_nb)
                        write_template(template, content_template)
  #                      compile_template(template)
                        # instanciation cheetah
                        a = instanciate_template(template, content)
                        return T.xml(a)
                    except Exception, e:
                        # impossible de recuperer le template demande
                        raise Exception, "template inexistant %s"%template
            except Exception:
                log.err("Echec du templating (soit la forme soit le contenu)")
                traceback.print_exc()
                # on bascule sur les affichages par defaut
                try:
                    return res_layout[type(data['content'])].place(data['content'])
                except KeyError:
                    # une vue par defaut
                    table = put_to_nevow_xml(str(data).split('\n'))# conversion en liste de tags nevow
                    return SpecialTableLayout(1).place(table)
                except TypeError:
                    table = put_to_nevow_xml(str(data).split('\n'))# conversion en liste de tags nevow
                    return SpecialTableLayout(1).place(table)
    table = put_to_nevow_xml(str(data).split('\n'))# conversion en liste de tags nevow
    return SpecialTableLayout(1).place(table)

def put_to_nevow_xml(_list):
    """ rend des elements nevow depuis une liste de string"""
    result = []
    for i in _list:
        result.append(T.div[i])
    return result
