# -*- coding: utf-8 -*-
""" Page principale de l'ead Version test"""

from twisted.web import util  # outil de redirection
from twisted.python import log
from os.path import join 
from os.path import isdir
from os import listdir
import xmlrpclib, socket
from nevow import loaders, tags as T, static, rend, inevow
from nevow.url import URL, here
from nevow.flat import flatten

from ead2.frontend.web.config import TEST_SOCKET_TIMEOUT, forbidden_actions
from config import STATIC_DIR, HTML_DIR, SCRIPT_DIR, STYLE_DIR
from ead2.lib.error import *
from ead2.config.config import AUTH_FORM_URL, AUTH_SERVER, MAGIC_NUMBER_TIMEOUT, AUTH_SERVER_ADDR, AUTH_FORM_PORT
from ead2.lib.libsecure import TransportEole

# les libs
from ead2.frontend.web.lib.tools import servers, _init_session, reset_session_params
from ead2.frontend.web.lib.tools import get_menu, initialize_ead_files, sso_alive
from ead2.frontend.web.lib.menu import Menu     # fragment renvoyant le menu
from ead2.frontend.web.lib.tabs import Tabs     # fragment pour les onglets de connexion aux serveurs
from ead2.frontend.web.lib.accueil import Accueil      # fragment centrale de l'accueil
from ead2.frontend.web.lib.action import Action      # fragment centrale pour les actions
from ead2.frontend.web.lib.admin import Admin     # fragment de presentation pour les actions add/del_server
from ead2.frontend.web.lib.error import ErrorPage # fragment pour les retours d'erreur
from ead2.frontend.web.lib.layout import WidgetLayout, TableLayout
from ead2.frontend.web.lib.formeol import ClassicForm

import traceback
import exceptions
from urllib import quote

class Page(rend.Page):
    """ page de routage de l'ead """

    docFactory = loaders.xmlfile(join(HTML_DIR, 'page.html'))

    def _set_config(self):
        """ charge les paramètres de
            configuration de l'interface
        """
        self.auth_server_address = AUTH_SERVER
        self.auth_server = xmlrpclib.ServerProxy(AUTH_SERVER)
        self.init_URL = ''

    def _set_language(self, request, session, ctx):
        """ gère la langue d'affichage, et le style
        """
        # on fixe la langue dans le contexte de page 
        ctx.remember(['fr'], inevow.ILanguages)
        lang = request.getCookie('language')
        if lang:
            session.language = lang
            ctx.remember([lang], inevow.ILanguages)
        if request.args != {}:
            # gestion de la feuille de style
            if request.args.has_key('style'):
                session.style = request.args['style'][0]
                request.args.pop('style')
                # (re)mise à jour du cookie
                request.addCookie('ead2style', session.style, path='/', max_age=3600*24*365) #1an

    def error_validate(self, message):
        """ renvoie un tag d'erreur """
        return T.xml("<div id='content'><font size='5' color='#ff0000'>%s</font></div>"%message)

    def validate_add_server(self, content, ctx):
        """ valide un ajout de serveur """
        if content.has_key('ip') and content.has_key('port') and content.has_key('servname') and content.has_key('login') and content.has_key('passwd') and content.has_key('language'):
            ip = content["ip"][0]
            port = content['port'][0]
            login = content['login'][0]
            passwd = content['passwd'][0]
            serv_name = content['servname'][0]
            language = content['language'][0]
        else: 
            return False, "il manque des paramètres dans la saisie"
        if '' in [ip, port, login, passwd, serv_name, language]:
            return False, "il manque des paramètres dans la saisie"
        server = 'https://%s:%s'%(ip,port)
        proxy = xmlrpclib.ServerProxy(server, transport=TransportEole())
        try:
            #retour de la forme (code_retour, key)
            #FIXME : Pas de retour pas d'exception si le serveur n'existe pas
            socket.setdefaulttimeout(TEST_SOCKET_TIMEOUT)
            return_code, key = proxy.register_frontend(login, passwd, language)
        except Exception, e:
            return False, "register error"
        if return_code == 0:
            try:
                servers().add_server('https://'+str(ip), str(port), str(serv_name), str(key))
            except ExistingServer:
                return False, "server %s-%s already exists" % (str(ip), str(port))
            except:
                #FIXME: erreur inconnu
                return False, "erreur inconnu lors de l'enregistrement"
        else:
            return False, "Echec de l ajout du serveur"
        return True, "Reussite de l'ajout du serveur"

    def validate_del_server(self, content, ctx):
        """valide une suppression de serveur """
        if content.has_key('id_server') and content.has_key('login') and content.has_key('passwd') and content.has_key('language'):
            id_serv = content["id_server"][0]
            login = content['login'][0]
            passwd = content['passwd'][0]
            language = content['language'][0]
        else:
            return False, "Il manque des informations dans la saisie."
        if '' in [id_serv, login, passwd, language]:
            return False, "Il manque des informations dans la saisie."
        try:
            url, port, comment, key = servers().get_server(id_serv)
        except UnknownServer:
            return False,"Problème de clé de serveur côté interface."
        if content.has_key('force'):
            if content['force'][0] == 'on':
                log.msg('On force la desinscription')
                try:
                    servers().del_server(id_serv)
                except UnknownServer:
                    return False, "Problème de clé de serveur côté interface."
                return True, "Réussite de la suppression du serveur."
        else:
            server = '%s:%s'%(url,port)
            proxy = xmlrpclib.ServerProxy(server, transport=TransportEole())
            try:
                return_code = proxy.unregister_frontend(key, login, passwd)
            except Exception, e:
                #FIXME, voir le message d'erreur
                return False, "Erreur de désinscription côté serveur de commande."
        #authentification locale a marchée
        if return_code == 0:
            try:
                servers().del_server(id_serv)
            except UnknownServer:
                return False, "Problème de clé de serveur côté interface."
        else:
            return False, "Problème de désinscription côté serveur de commande."
        return True, "Réussite de la suppression du serveur"

    def logout(self, ctx):
        """ deconnecte un utilisateur"""
        session = inevow.ISession(ctx)
        reset_session_params(session)
        session.magic_number = None
        if sso_alive():
            url = URL.fromContext(ctx).clear().parentdir()
            url = "%s/logout/?service=%s"%(AUTH_FORM_URL,url)
        else:
            url = URL.fromContext(ctx).clear().parentdir()
        return util.Redirect(url)
                

    def validate(self, _type, arguments, ctx):
        """
        validation des actions de la page de base
        :_type: type d'action : add_server, del_server
        :arguments: arguments de la requête {'variabe':valeur}
        """
        session = inevow.ISession(ctx)
        dico_valid = {'add_server':self.validate_add_server,
                        'del_server':self.validate_del_server,
                        'logout':self.logout}
        return_type, msg = dico_valid[_type[0]](arguments, ctx)
        session.validate_error = True
        session.validate_msg = msg

    def locateChild(self, ctx, segments):
        """ gère le routage des pages """
        request = inevow.IRequest(ctx)
        session = inevow.ISession(ctx)
        # on initialise les param de session
        session = _init_session(session, request)
        child, remainingSegments = rend.Page.locateChild(self, ctx, segments)
        self.action = None
        if 'add_server' in segments:
            reset_session_params(session)
            if 'form' in request.args.keys():
                _type = request.args['form']
                self.validate(_type, request.args, ctx)
                return self, ()
            self.action = 'add_server'
            return self, ()
        if 'del_server' in segments:
            reset_session_params(session)
            if 'form' in request.args.keys():
                _type = request.args['form']
                self.validate(_type, request.args, ctx)
                return self, ()
            self.action = 'del_server'
            return self, ()
        self._set_config()
        self.init_URL = str(request.URLPath()).split('?')[0]
        # on gère la langue d'affichage
        self._set_language(request, session, ctx)
        if child:
            if child == self:
                reset_session_params(session)
            return child, remainingSegments
        elif request.args != {}:
            log.err("arguments non considere: ")
            log.err(request.args)

    def child_logout(self, ctx):
        """ deloggue l'utilisateur connecté """
        return self.logout(ctx)
            
    def child_connect(self, ctx):
        """ child renvoyant vers une page serveur (controle SSO)"""
        return ConnectedPage()

    def child_image(self, ctx):
        """chemin image"""
        return static.File(join(STATIC_DIR, 'image'))

    def child_script(self, ctx):
        """chemin script"""
        return static.File(join(STATIC_DIR, 'script'))

    def child_style(self, ctx):
        """chemin style"""
        return static.File(join(STATIC_DIR, 'style'))
    
    def render_icon(self, ctx, data):
        """ renvoit l'icone de la page """
        return T.link(_type="image/x-icon", href="/image/eole_icone.ico", rel="shortcut icon")
        
    def render_script(self, ctx, data):
        """ rend les imports de scripts """
        scripts = [T.script(_type="text/javascript", src="/script/nested.js", media="screen"),
                    T.script(_type="text/javascript", src="/script/formeol.js", media="screen"),
                    T.script(_type="text/javascript", src="/script/message_box.js", media="screen"),
                    T.script(_type="text/javascript", src="/script/ajax.js", media="screen"),
                    T.script(_type="text/javascript", src="/script/main.js", media="screen")
                    ]
        return scripts
    
    def render_style(self, ctx, data):
        """rend les imports de css """
        session = inevow.ISession(ctx)
        style = session.style
        links = [
                    T.link(_type="text/css", rel="stylesheet", href="/style/choice/%s"%style, media="screen"),
                    T.link(_type="text/css", rel="stylesheet", href="/style/ead.css", media="screen"),
                    T.link(_type="text/css", rel="stylesheet", href="/style/tabs.css", media="screen"),
                    T.link(_type="text/css", rel="stylesheet", href="/style/diode.css", media="screen"),
                    T.link(_type="text/css", rel="stylesheet", href="/style/message_box.css" , media="screen"),
                    T.link(_type="text/css", rel="stylesheet", href="/style/formeol.css" , media="screen")
                ]
        return links

    def render_tabs(self, ctx, data):
        """onglets en haut de page (liste des serveurs)
        """
        # on ref les onglets pour pouvoir les commander depuis un autre fragment en python.
        tabs = Tabs()
        tabs.page = self
        return tabs

    def render_main(self, ctx, data):
        """section principale de la page """
        session = inevow.ISession(ctx)
        # si on revient d'une execution
        if session.validate_error:
            main = self.error_validate(session.validate_msg)
            session.validate_error = None
            session.validate_msg = None
        else:
            main = Accueil() 
        if self.action is not None:
            # on réaffiche la dernière action sauf si elle ne peut pas se présenter sans infos préalable
            log.msg("Cette action est reaffichable ")
            main = Admin(self.action)
        return T.div(_id='content')[main]

    def render_menu(self, ctx, data):
        return '' 

    def render_entete(self, ctx, data):
        """ rend l'entete de la page """
        return '' 

    def render_above_part(self, ctx, data):
        """ rend l'entete de la page """
        return T.center[T.img(src='/image/logo_small.png', style="vertical-align:bottom")]
    
    def data_admin(self, ctx, data):
        """ data pour la partie admin"""
        # construction d'une url pour le logout
        session = inevow.ISession(ctx)
        server_nb = session.current_server
        url = URL.fromContext(ctx).clear()
        url = url.parentdir().parentdir()
        logout_url = url.sibling('logout').child('')
        accueil = ["Accueil", url]
        logout = ["Déconnexion", logout_url]
        add_url = url.sibling('add_server').child('')
        del_url = url.sibling('del_server').child('')
        reload_conf = ["Recharger", "javascript:location.reload();"]
        add_server = ["Ajouter Serveur", add_url]#"javascript:call_content('./ajax', 'add_server');"]
        del_server = ["Supprimer Serveur", del_url]#"javascript:call_content('./ajax','del_server');"]
        data = [accueil, reload_conf, add_server, del_server, logout]
        return data

    def render_admin(self, ctx, data):
        """menu de gestion des serveurs
        """
        rightside = T.div[T.div(_id='rightside_top')[
						 T.div(_class='admin_toggle_link')[
                         T.a(href="javascript:ToggleAdmin('rightside_top_link','rightside_bottom')")[
                         T.h1["Administration"],
                             T.img(_id='rightside_top_link', src='/image/opened.gif')]]]]
        admin = T.div(_id='rightside_bottom', valign='top', _class='admin_open')
        tag = T.p()
        for element in data:
            tag.children.append(T.a(href=element[1])[T.span[element[0]], T.br()])
        admin.children.append(tag)
        admin.children.append(T.div(render=T.directive('designlist')))
        rightside.children.append(admin)
        return rightside

    def render_designlist(self, ctx, data):
        """
        création de la liste des design existants
        pour mettre dans la balise select
        """
        server_nb = inevow.ISession(ctx).current_server
        options = []
        for style in inevow.ISession(ctx).styles :
            if style == inevow.ISession(ctx).style :
                # présélection du style courant
                options.append(T.option(name=style, selected='selected')[style])
            else:
                options.append(T.option(name=style)[style])
        return T.form(method='post')[T.select(name='style', onchange="submit()")[options]]

class ConnectedPage(Page):
    """
    page d'affichage de la section serveur 
    """
    docFactory = loaders.xmlfile(join(HTML_DIR, 'server.html'))

    def __init__(self):
        Page.__init__(self)
        self.request_dict = {'ajax': self.ajax, 'error':self.error}

    def ajax_redirect(self, url):
        """ outil de redirection de requete ajax """
        return "redirect:%s"%str(url), ()
    
    def redirect(self, url):
        """ outil de redirection classique """
        return util.Redirect(str(url)), ()

    def make_url(self, **args):
        url = self.init_URL
        for key, value in args.items():
            url = url.add(key, value)
        return url
    
    def ajax(self):
        session = inevow.ISession(self.ctx)
        request = inevow.IRequest(self.ctx)
        if 'action' in request.args.keys():
            session.current_action = request.args['action'][0]
            return Action(session.current_action, session.menu, self.ctx), ()
    
    def error(self):
        session = inevow.ISession(self.ctx)
        reset_session_params(session)
        url = self.init_URL.parentdir()
        return self.redirection(url)

    def auth_redirect(self, server_nb):
        session = inevow.ISession(self.ctx)
        session.current_server = server_nb
        session.menu = None
        session.connect_to = server_nb
        if not sso_alive():
            url = self.make_url(server=server_nb, local_auth='true')
            return self.redirection(url)
        #on est dirigé vers le serveur d'auth pour obtenir un magic_nb pour le backend correspondant
        url = "%s/?service=%s"%(AUTH_FORM_URL, self.init_URL)
        return self.redirection(url)

    def locateChild(self, ctx, segments):
        """ localise les childs permettant de répondre à la requête
            1- effectue le contrôle Sso
            2- effectue le traitemant de requête (gestion des requetes ajax et des retours de 
                                        formulaire pour les actions simples)
        """
        self.ctx = ctx
        child, remainingSegments = rend.Page.locateChild(self, ctx, segments)
        if child:
            return child, remainingSegments
        request = inevow.IRequest(ctx)
        session = inevow.ISession(ctx)
        self.init_URL = URL.fromContext(ctx).clear().child('')
        if 'ajax' in segments:
            self.redirection = self.ajax_redirect
        else:
            self.redirection = self.redirect
        if request.args != {}:
            """1- début du traitement SSo
                a-connexion au serveur d'auth
                b-recupération du ticket
                c-envoi du ticket au backend
                d-validation de l'authentification
                e-vérification de l'intégrité des variables de session
            """
            session.menu = None
            #doit on mettre un magic_number dans la session
            put_magic = False
            #si ticket d'application présent (authentification SSO)
            #cas où l'on revient de l'authentification et qu'on a les parametres dans l'URL
            server_nb = None
            if not request.args.has_key('server'):
                if hasattr(session,'connect_to'):
                    server_nb = session.connect_to
                    delattr(session,'connect_to')
            else:
                server_nb = request.args['server'][0]
            if server_nb is not None:
                try:
                    #on vérifie que le numero de serveur est valide
                    serv = servers().get_server(server_nb)
                    url, port, comment, key = serv
                except UnknownServer:
                    log.err("serveur non reference :")
                    return self.request_dict['error']()
                if serv is not None:
                    ead_host = "%s:%s" % (url, port)
                    session.ead_server = xmlrpclib.ServerProxy(ead_host)
                    try:
                        session.ead_server.is_alive().strip()
                    except:
                        return self.request_dict['error']()
                else:
                    log.err("numéro de serveur invalide")
                    #si le numero de serveur est invalide, on se redirige sur l'EAD
                    return self.request_dict['error']()
                if request.args.has_key('ticket'):
                    app_ticket = request.args['ticket'][0]
                    #on va recuperer un magic_number
                    try:
                        url = self.init_URL
                        magic_nb_tmp = session.ead_server.get_magic_number(key, app_ticket, str(url), 
                                                                           session.language)
                    except socket.error:
                        log.err("socket error")
                        return self.request_dict['error']()
                    #on dit qu'on devra mettre un cookie
                    put_magic = True 
                #cas où l'on revient de l'authentification, mais locale
                #si authentification locale, on met un cookie avec magic_number
                #meme si il est faux, il ne passera pas quand il sera transmis au backend     
                elif request.args.has_key('local') and request.args.has_key('magic_nb'):
                    magic_nb_tmp = request.args['magic_nb'][0]
                    ead_host = "%s:%s" % (url, port)
                    session.ead_server = xmlrpclib.ServerProxy(ead_host)
                    request.args.pop('local')
                    request.args.pop('magic_nb')
                    put_magic = True
                #si numero de serveur (ie: on clique pour se connecter a un backend)
                elif server_nb != session.current_server:
                    #si on se connecte a un "nouveau" serveur
                    return self.auth_redirect(server_nb)
                elif request.args.has_key('local_auth'):
                    if session.magic_number is None:
                        # authentification locale, le SSo est tombé
                        if not request.args.has_key('login'):
                            return FormPage(), ()
                        else:
                            password = request.args['passwd'][0]
                            username = request.args['login'][0]
                            # l'authentification sso a échoué : on passe en authentification locale
                            url, port, comment, key = servers().get_server(server_nb)
                            ead_server = xmlrpclib.ServerProxy("%s:%s"%(url,port))
                            try:
                                magic_nb_tmp = ead_server.local_authentification(key, username,
                                                                password, True, session.language)
                            except:
                                # erreur de communication avec le backend
                                return self.request_dict['error']()
                            if not magic_nb_tmp: # echec d'authentification
                                return FormPage(False), ()
                    else:
                        magic_nb_tmp = session.magic_number
                    if magic_nb_tmp is not None:
                        session.connect_to = server_nb
                        url = self.make_url(local='1', magic_nb=magic_nb_tmp, server=server_nb)
                        return self.redirection(url)
            else:
                #si il manque un parametre, on se redirige sur l'EAD
                return self.request_dict['error']()
            if put_magic:
                put_magic = False
                if magic_nb_tmp:
                    myurl = self.make_url(server=server_nb)
                    #on met notre magic number dans l'objet session
                    session.magic_number = magic_nb_tmp
                    #on se redirige sur nous meme, mais on a maintenant un magic_number
                    initialize_ead_files(session, request)
                    return self.redirection(myurl)
        #pas de parametre dans l'URL, on redirige
        else:
            return self.request_dict['error']()
        """ vérification de l'intégrité des variables de session
            a-controle magicnb et existence d'un serveur courant
            b-récupération du menu et vérification de la réussite
        """
 #       magic_nb = request.getCookie('magic')
        magic_nb = session.magic_number
        if not magic_nb or not session.current_server:
            return self.request_dict['error']()
        else:
            if session.menu is None:
                get_menu(ctx)
            # si l on arrive pas a charger le menu, on retente et sinon on renvoit la page appropriee
            if not session.menu:
                return self.request_dict['error']()
        """ 2- récupération des variables de session 
            :ajax: appel de contenu via ajax
            :execution: retour de formulaire
        """
        if 'ajax' in segments:
            return self.request_dict['ajax']()
        ## contact d'une iframe pour le chargement de fichier si elle envoie le fichier on répond
        if 'file_download' in segments:
            if request.args.has_key(request.args['filename'][0]) and request.args.has_key('action'):
                ## on utilise quote de urllib pour passer à travers le tube xmlrpc
                request.args[request.args['filename'][0]][0] = quote(request.args[request.args['filename'][0]][0])
                return Action(request.args['action'][0], session.menu, self.ctx),()
            return '', ()
        return self, ()

    def render_script(self, ctx, data):
        """ rend les imports javascripts """
        session = inevow.ISession(ctx)
        scripts = [T.script(_type="text/javascript", src="/script/nested.js", media="screen"),
                    T.script(_type="text/javascript", src="/script/formeol.js", media="screen"),
                    T.script(_type="text/javascript", src="/script/message_box.js", media="screen"),
                    T.script(_type="text/javascript", src="/script/ajax.js", media="screen"),
                    T.script(_type="text/javascript", src="/script/json.js", media="screen"),
                    T.script(_type="text/javascript", src="/script/formulaire.js", media="screen"),
                    T.script(_type="text/javascript", src="/script/main.js", media="screen")
                    ]
        if session.module_name is not None: # import des scripts specifiques au module en cour de traitement
            scriptdir = join(SCRIPT_DIR, session.module_name)
            if isdir(scriptdir):
                script_list = listdir(scriptdir)
                for script in script_list:
                    if script[-2:] == 'js':
                        scripts.append(T.script(_type="text/javascript", src="/script/%s/%s"%(session.module_name, script),
                                        media="screen"))
        return scripts
    
    def render_style(self, ctx, data):
        """rend les imports de css """
        session = inevow.ISession(ctx)
        style = session.style
        links = [   T.link(_type="text/css", rel="stylesheet", href="/style/choice/%s"%style , media="screen"),
                    T.link(_type="text/css", rel="stylesheet", href="/style/ead.css" , media="screen"),
                    T.link(_type="text/css", rel="stylesheet", href="/style/tabs.css", media="screen"),
                    T.link(_type="text/css", rel="stylesheet", href="/style/diode.css", media="screen"),
                    T.link(_type="text/css", rel="stylesheet", href="/style/message_box.css" , media="screen"),
                    T.link(_type="text/css", rel="stylesheet", href="/style/actions.css" , media="screen"),
                    T.link(_type="text/css", rel="stylesheet", href="/style/formeol.css" , media="screen")
                ]
        if session.module_name is not None: # import des styles specifiques au module en cour de traitement
            styledir = join(STYLE_DIR, session.module_name)
            if isdir(styledir):
                style_list = listdir(styledir)
                for style in style_list:
                    if style[-3:] == 'css':
                        links.append(T.link(_type="text/css", rel="stylesheet", 
                                href="/style/%s/%s"%(session.module_name, style), media="screen"))
        return links


    def render_menu(self, ctx, data):
        """menu d'actions du serveur choisi vide par defaut
        """
        inevow.ISession(ctx).menu
        menu = Menu(ctx) 
        return menu
    
    def render_entete(self, ctx, data):
        """ rend l'entete de la page """
        return ''

    def render_main(self, ctx, data):
        """ rend le corp de la page """
        session = inevow.ISession(ctx)
        if session.current_action is not None and session.current_action not in forbidden_actions:
            try:
                a = Action(session.current_action, session.menu, self.ctx)
            except:
                a = Action('main_status', session.menu, self.ctx)
        else:
            a = Action('main_status', session.menu, self.ctx)
        return T.div(_id='content')[a]
    
    def render_admin(self, ctx, data):
        """menu de gestion des serveurs
        """
        rightside = T.div[T.div(_id='rightside_top')[
						 T.div(_class='admin_toggle_link')[
                         T.a(href="javascript:ToggleAdmin('rightside_top_link','rightside_bottom')")[
                         T.h1["Administration"],
                             T.img(_id='rightside_top_link', src='/image/closed.gif')]]]]
        admin = T.div(_id='rightside_bottom', valign='top', _class='admin_close')
        tag = T.p()
        for element in data:
            tag.children.append(T.a(href=element[1])[T.span[element[0]], T.br()])
        admin.children.append(tag)
        admin.children.append(T.div(render=T.directive('designlist')))
        rightside.children.append(admin)
        return rightside

    def data_admin(self, ctx, data):
        """ data pour la partie admin"""
        session = inevow.ISession(ctx)
        server_nb = session.current_server
        url = URL.fromContext(ctx).clear()
        url = url.parentdir().parentdir()
        accueil = ["Accueil", url]
        logout_url = "%slogout/"%url
        add_url = "%sadd_server/"%url
        del_url = "%sdel_server/"%url
        logout = ["Déconnexion", logout_url]#"javascript:call_outil('%s', 'logout')"%server_nb]#redirect_url]
        reload_conf = ["Recharger", "javascript:location.reload();"]
        add_server = ["Ajouter Serveur", add_url]#"javascript:call_outil('%s', 'add_server')"%server_nb]
        del_server = ["Supprimer Serveur", del_url]#"javascript:call_outil('%s', 'del_server');"%server_nb]
        data = [accueil, reload_conf, add_server, del_server, logout]
        return data

class FormPage(Page):
    """ page d'authentification locale """

    def __init__(self, first=True):
        Page.__init__(self)
        self.first = first

    def render_main(self, ctx, data):
        """ rend un formulaire dans la partie centrale de la page """
        form = ClassicForm(name="local_auth", action="")
        form.page = self
        form.set_field_layout(TableLayout(3))
        form.set_layout(TableLayout(1))
        form.add_field('login', "Login", required=True, default="")
        form.add_passwd_field('passwd', "Mot de passe", required=True, default='')
        form.add_submit("Valider")
        err = ''
        if not self.first:
            err = T.h4[T.font(color='#dd3333')["Echec lors de l'authentification, réessayez"]]
        return T.div(_class="formeol")[T.h1["Authentification locale"], err,form]
    
